#!/usr/bin/sage
from EllipticCurve import EllipticCurveModRing
from SampleFunctions import createEllipticCurveAdditionExample

class ModularAddition(EllipticCurveModRing):
    p = 11
    xMin = 0
    xMax = p

    def createEllipticCurve(self):
        result = createEllipticCurveAdditionExample(self, showLines=True)
        self.out.indent("Counting group order starting with x=1 and using point doubling.")

        allPoints = []
        n = 1 # count of the group order, starting with the O point
        p0 = (1, self.curveFunction(1))
        p = p0
        while p != self.POINT_AT_INFINITY:
            allPoints.append(p)
            p = self.addPoints(p, p0)
#            result += self.point(p, "p%d" % n)
            n += 1
        self.out.unIndent()
        self.out.put("Got point at inifinity. Found group order of %d\n\n" % n)


        self.out.indent("Now, finding G such that nG=O:")
        for p in allPoints:
            O = self.doubleAndAdd(p, n)
            self.out.put("%d x %s = %s" % (n, p, O), self.out.LOG_LEVEL_VERBOSE)
            self.out.put('')
        self.out.unIndent()

        return result

if __name__ == "__main__":
    result = ModularAddition()
    result.run()
