#!/usr/bin/sage
from JobOutput import JobOutput

import sys
from time import time

print('Loading Sage...')
from sage.all import *

class SageBase:

    out = None #JobOutput object
    timestamp = False
    convertedArguments = []

    INDENT = 15
    FIXED_WIDTH = 75

    def __init__(self, out=None):
        self.pointLabels = {} # workaround for caching issue
        if out == None:
            out = JobOutput()
            out.disableLogFile()
        self.out = out

    def timer(self):
        currentTime = time()
        if self.timestamp:
            self.out.put('- this step took %d second(s)\n' % (currentTime - self.timestamp))
        self.timestamp = currentTime

    def printPoint(self, label, point, logLevel=None):
        try:
            pointString = "(%s, %s)" % point
        except:
            try:
                pointString = "(%.2f, %.2f)" % point
            except:
                try:
                    pointString = "(%Ld, %Ld)" % point
                except:
                    pointString = "(%s, %s)" % point
        self.printNumber(label, pointString, logLevel)

    def printNumber(self, label, number, logLevel=None):
        output = ('%-' + str(self.INDENT) + 's' + str(number)) % (label + ': ')
        if len(output) > self.FIXED_WIDTH:
            output = ('%s:%' + str(self.FIXED_WIDTH - len(label) - 1) + 's') % (label, number)
        if len(output) > self.FIXED_WIDTH:
            output = ('%s:\n%' + str(self.FIXED_WIDTH) + 's') % (label, number)
        self.out.put(output, logLevel)

    def run(self):
        self.convertArgsToSageValues()

        self.out.put('Starting %s...' % sys.argv[0])
        self.doRunSteps()
        self.out.put('Done.')

    def convertArgsToSageValues(self):
        for i in range(1, len(sys.argv)):
            self.convertedArguments.append(sage_eval(sys.argv[i]))

    def doRunSteps(self):
        raise 'Illegal to call base class!!'

    def assertEqual(self, a, b):
        if a == b:
            self.out.put("Success!")
        else:
            self.out.put("FAIL")
