#!/usr/bin/sage
from EllipticCurve import EllipticCurveModRing, EllipticCurveStandard


from sage.all import *

class SmallExampleNonModulo(EllipticCurveStandard):
    # curve/field characteristics
    a = -4
    b = 4
    p = 7

    # graph output
    xMax = 6
    xMin = -3


    def createEllipticCurve(self):
        allPoints = []

        result = self.ellipticCurve()

        n = 1 # count of the group order, starting with the O point
        x = 2
        y = self.curveFunction(x)
        p0 = (x, y)
        result += self.point(p0, "p")


        self.out.indent('Checking that (p + p + p) + p == 2 x (p + p)')
        p = p0
        self.printPoint("p", p)
        ppp_p = self.addPoints(self.addPoints(self.addPoints(p, p), p), p)
        result += self.point(ppp_p, "\n\\...........(p+p+p)+p")
        pp = self.addPoints(p, p)
        result += self.point(pp, "p+p")
        pp_pp = self.addPoints(pp, pp)
        result += self.point(pp_pp, "2(p+p)")

        self.printPoint("(p + p + p) + p", ppp_p)
        self.printPoint("2 x (p + p)", pp_pp)
        self.assertEqual(ppp_p, pp_pp)
        self.out.unIndent()




        self.out.indent('Checking that (p + q) + r == p + (q + r)')
        p = p0
        self.printPoint("p", p)
        q = (x + 1, self.curveFunction(x + 1))
        result += self.point(q, "q")

        r = (x + 2, self.curveFunction(x + 2))
        result += self.point(r, "r")

        self.out.indent('p + (q + r) = ...')
        qr = self.addPoints(q, r)
        result += self.point(qr, "\n\\.....q+r")
        self.printPoint("q + r", qr)

        p_qr = self.addPoints(p, qr)
        result += self.point(p_qr, "p+(q+r)")
        self.printPoint("p +(q + r)", p_qr)
        self.out.unIndent()

        self.out.indent('(p + q) + r = ...')
        pq = self.addPoints(p, q)
        result += self.point(pq, "p+q")
        self.printPoint("p + q", pq)

        pq_r = self.addPoints(pq, r)
        result += self.point(pq_r, "\n\\.......(p+q)+r")
        self.printPoint("(p + q) + r", pq_r)
        self.out.unIndent()

        self.assertEqual(pq_r, p_qr)

        self.out.unIndent()

        return result


if __name__ == "__main__":
    result = SmallExampleNonModulo()
    result.run()
