#!/usr/bin/sage
from EllipticCurve import EllipticCurveStandard
from SampleFunctions import showRelatedPoints

class SPEKE(EllipticCurveStandard):
    '''
    a nice example of the SPEKE protocol in a visually accessible (non-modulo) format
    '''
    xMax = 4
    a = -5
    b = 5

    def createEllipticCurve(self):

        result = self.ellipticCurve()

        self.out.indent("Alice and Bob are both generating h(pw)...")
        self.h_pw = (0, self.curveFunctionMinus(0))
        result += self.point(self.h_pw, "H(pw)")
        self.printPoint("h(pw)", self.h_pw)
        self.out.unIndent()

        self.out.indent("Alice is generating a private key using a random x=2.5...")
        self.alice = (2.5, self.curveFunctionMinus(2.5))
        self.printPoint("Alice's private key", self.alice)

        self.a_h_pw = self.addPoints(self.alice, self.h_pw)
        self.printPoint("Alice generated A = a.h(pw)", self.a_h_pw)
        self.bob = (1.75, self.curveFunctionMinus(1.75))
        self.b_h_pw = self.addPoints(self.bob, self.h_pw)
        self.k = self.addPoints(self.alice, self.b_h_pw)
        self.out.unIndent()
        return result

class BobStep1(SPEKE):
    def createEllipticCurve(self):
        # a + H(pw) = a.H(pw)
        result = SPEKE.createEllipticCurve(self)
        result += self.point(self.alice, "ka")
        result += self.point(self.a_h_pw, "A")
        result = showRelatedPoints(self, self.alice, self.a_h_pw, result)
        return result

class BobStep2(SPEKE):
    def createEllipticCurve(self):
        result = SPEKE.createEllipticCurve(self)

        result += self.point(self.alice, "ka")
        result += self.point(self.bob, "kb")
        result += self.point(self.a_h_pw, "A")
        result += self.point(self.k, "K")
        result += self.point(self.k, "kb.A")

        result = showRelatedPoints(self, self.alice, self.a_h_pw, result)
        result = showRelatedPoints(self, self.bob, self.k, result)
        result = showRelatedPoints(self, self.a_h_pw, self.k, result)
        return result

class AliceStep1(SPEKE):
    def createEllipticCurve(self):
        # b + H(pw) = b.H(pw)
        result = SPEKE.createEllipticCurve(self)
        result += self.point(self.bob, "kb")
        result += self.point(self.b_h_pw, "B")
        result = showRelatedPoints(self, self.bob, self.b_h_pw, result)
        return result

class AliceStep2(SPEKE):
    def createEllipticCurve(self):
        result = SPEKE.createEllipticCurve(self)

        result += self.point(self.bob, "kb")
        result += self.point(self.b_h_pw, "B")
        result += self.point(self.alice, "ka")

        result += self.point(self.k, "K")
        result += self.point(self.k, "ka.B")

        result = showRelatedPoints(self, self.bob, self.b_h_pw, result)
        result = showRelatedPoints(self, self.b_h_pw, self.k, result)
        result = showRelatedPoints(self, self.alice, self.k, result) # extraneous, just to show the through-line
        return result


if __name__ == "__main__":
    AliceStep1().run()
    AliceStep2().run()
    BobStep1().run()
    BobStep2().run()
