#!/usr/bin/sage
from EllipticCurve import EllipticCurveModRing

class SmallSwuSample(EllipticCurveModRing):
    p = 11
    t = 42
    xMin = 0
    xMax = p

    def createEllipticCurve(self):
        result = self.ellipticCurve()
        self.out.indent("Performing SWU algorithm starting at value %d." % self.t)

        tH = self.swuHash(self.t)
        result += self.point(tH, "H(t), t=%d" % self.t)
        self.out.unIndent()
        return result

class BigSwuSample(SmallSwuSample):
    p = 6277101735386680763835789423207666416083908700390324961279
    n = 6277101735386680763835789423176059013767194773182842284081
    a = -3
    b = 0x64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1
    Gx = 0x188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012
    Gy = 0x07192b95ffc8da78631011ed6b24cdd573f977a11e794811

    def createEllipticCurve(self):
        G = (self.Gx, self.Gy)
        shouldBeInfinity = self.doubleAndAdd(G, self.n)
        self.printNumber("nG (should be PAI)", shouldBeInfinity)

        self.out.indent("Performing SWU algorithm starting at point %d." % self.t)
        tH = self.swuHash(self.t)

        self.out.unIndent()

if __name__ == "__main__":
    SmallSwuSample().run()
#    BigSwuSample().run()
