#!/usr/bin/sage -python
from EllipticCurve import EllipticCurveModRing
from sage.all import gcd

# NIST p-192
class DoubleAndAdd3p(EllipticCurveModRing):
    a = -3
    b = 0x64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1

    p = 6277101735386680763835789423207666416083908700390324961279
    n = 6277101735386680763835789423176059013767194773182842284081

    Gx = 0x188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012
    Gy = 0x07192b95ffc8da78631011ed6b24cdd573f977a11e794811



    def createEllipticCurve(self):
        gYOnCurvePlus = self.curveFunction(self.Gx)
        gYOnCurveMinus = self.curveFunctionMinus(self.Gx)
        assert(self.Gy == gYOnCurvePlus or self.Gy == gYOnCurveMinus)

        nG = self.doubleAndAdd((self.Gx, self.Gy), self.n)
        assert(nG == self.POINT_AT_INFINITY)

        # Test if arbitrary points are on the curve
        GxPrime = self.Gx + 1
        print "gcd'ing self.Gx + 1: %s" % gcd(GxPrime, self.p)
        GyPrime = self.curveFunction(GxPrime)
        nGPrime = self.doubleAndAdd((GxPrime, GyPrime), self.n)
        assert(nGPrime == self.POINT_AT_INFINITY)

        GyPrime = self.curveFunctionMinus(GxPrime)
        nGPrime = self.doubleAndAdd((GxPrime, GyPrime), self.n)
        assert(nGPrime == self.POINT_AT_INFINITY)

        GxPrime = self.Gx + 2
        print "gcd'ing self.Gx + 2: %s" % gcd(GxPrime, self.p)
        GyPrime = self.curveFunction(GxPrime)
        nGPrime = self.doubleAndAdd((GxPrime, GyPrime), self.n)
        assert(nGPrime == self.POINT_AT_INFINITY)

        GyPrime = self.curveFunctionMinus(GxPrime)
        nGPrime = self.doubleAndAdd((GxPrime, GyPrime), self.n)
        assert(nGPrime == self.POINT_AT_INFINITY)

        # explore a neighboring group on the curve
        GxPrime = self.Gx - 1
        GyPrime = self.curveFunction(GxPrime)
        n = self.p - self.n + 1 # should be all the remaining points not in <G> right?
        nGPrime = self.doubleAndAdd((GxPrime, GyPrime), n)
        assert(nGPrime != self.POINT_AT_INFINITY)
        print "found point on curve that is NOT in <G>"
        # I think the problem is that self.p is not actually prime; some points are divisible.
        print "gcd'ing self.Gx - 1: %s" % gcd(GxPrime, self.p)
#        assert( > 1)

if __name__ == "__main__":
    s = DoubleAndAdd3p()
    s.run()
